--DROP TABLE power_state;
CREATE TABLE power_state
(
  powerstate  bigint NOT NULL,
  description character varying(255),
  CONSTRAINT power_state_pkey PRIMARY KEY (powerstate)
);
--DROP TABLE device_type;
CREATE TABLE device_type
(
  devicetype  bigint NOT NULL,
  description character varying(255),
  CONSTRAINT device_type_pkey PRIMARY KEY (devicetype)
);
--DROP TABLE device_status;
CREATE TABLE device_status
(
  globalstatus bigint NOT NULL,
  description character varying(255),
  CONSTRAINT global_status_pkey PRIMARY KEY (globalstatus)
);
--DROP TABLE location_type;
CREATE TABLE location_type
(
  locationtype  bigint NOT NULL,
  description 	character varying(255),
  CONSTRAINT location_type_pkey PRIMARY KEY (locationtype)
);
--DROP TABLE management_type;
CREATE TABLE management_type
(
  managementtype  bigint NOT NULL,
  description 	  character varying(255),
  CONSTRAINT management_type_pkey PRIMARY KEY (managementtype)
);

--DROP TABLE software_type;
CREATE TABLE software_type
(
  softwaretype    bigint NOT NULL,
  description 	  character varying(255),
  CONSTRAINT software_type_pkey PRIMARY KEY (softwaretype)
);
--DROP TABLE group_type;
CREATE TABLE group_type
(
  grouptype    	  bigint NOT NULL,
  description 	  character varying(255),
  CONSTRAINT group_type_pkey PRIMARY KEY (grouptype)
);
--DROP TABLE infrastructure_device;
CREATE TABLE infrastructure_device
(
  deviceid    	bigint NOT NULL,
  model 	   	character varying(128) NOT NULL,
  identifier   	character varying(128) NOT NULL,
  assettag 	   	character varying(128),
  devicetype   	bigint NOT NULL,
  powerstate   	bigint NOT NULL,
  globalstatus 	bigint NOT NULL,
  systemid 		bigint NOT NULL,
  CONSTRAINT infrastructure_device_pkey PRIMARY KEY (deviceid),
  CONSTRAINT fk_device_type FOREIGN KEY (devicetype)
      REFERENCES device_type (devicetype) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_device_global_status FOREIGN KEY (globalstatus)
      REFERENCES device_status (globalstatus) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_device_power_state FOREIGN KEY (powerstate)
      REFERENCES power_state (powerstate) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);

--DROP TABLE device_management;
CREATE TABLE device_management
(
  managementid  bigint NOT NULL,
  deviceid      bigint NOT NULL,
  ipaddress 	 character varying(30) NOT NULL,
  macaddress 	 character varying(30) NOT NULL,
  managementurl  character varying(1024) NOT NULL,
  managementtype bigint NOT NULL,
  CONSTRAINT device_management_pkey PRIMARY KEY (managementid ),
  CONSTRAINT fk_device_management FOREIGN KEY (deviceid)
      REFERENCES infrastructure_device (deviceid) MATCH SIMPLE ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_device_management_type FOREIGN KEY (managementtype)
      REFERENCES management_type (managementtype) MATCH SIMPLE ON UPDATE CASCADE ON DELETE CASCADE
);
--DROP TABLE infrastructure_group;
CREATE TABLE infrastructure_group
(
  groupid 				bigint 	NOT NULL,
  groupname 			character varying(255),
  groupdescription 		character varying(255),
  grouptype 			bigint NOT NULL,
  CONSTRAINT device_group_pkey PRIMARY KEY (groupid ),
  CONSTRAINT fk_device_group FOREIGN KEY (groupid)
      REFERENCES infrastructure_device (deviceid) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_device_group_type FOREIGN KEY (grouptype)
      REFERENCES group_type (grouptype) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);
--DROP TABLE group_device_association;
CREATE TABLE group_device_association
(
  groupid 		  		bigint 	NOT NULL,
  deviceid 	      		bigint    NOT NULL,
  locationname 	  		character varying(255),
  locationidentifier 	character varying(128),
  locationtype 		  	bigint, 	
  CONSTRAINT device_group_association_pkey PRIMARY KEY ( groupid, deviceid ),
  CONSTRAINT fk_device_association_group FOREIGN KEY (groupid)
      REFERENCES infrastructure_group (groupid) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_device_association_device FOREIGN KEY (deviceid)
      REFERENCES infrastructure_device (deviceid) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_device_association_location_type FOREIGN KEY (locationtype)
      REFERENCES location_type (locationtype) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);
--DROP TABLE credentials_type;
CREATE TABLE credentials_type
(
  credentialstype    	  bigint NOT NULL,
  description 	  		  character varying(255),
  CONSTRAINT credentials_type_pkey PRIMARY KEY (credentialstype)
);
--DROP TABLE credentials;
CREATE TABLE credentials
(
  credentialsid  bigint NOT NULL,
  credentialsparamters  character varying(2048),
  connection_profile_id  bigint,
  credentialstype bigint NOT NULL,
  CONSTRAINT credentials_pkey PRIMARY KEY (credentialsid),
  CONSTRAINT fk_credentials_type FOREIGN KEY (credentialstype)
      REFERENCES credentials_type (credentialstype) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_credentials_connection_profile FOREIGN KEY (connection_profile_id)
      REFERENCES connection_profile (connection_profile_id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);
--DROP TABLE device_credentials_map;
CREATE TABLE device_credentials_map
(
  deviceid  		bigint NOT NULL,
  credentialsid  	bigint NOT NULL,
  --operation type
  CONSTRAINT credentials_device_map_pkey PRIMARY KEY (deviceid, credentialsid),
  CONSTRAINT fk_credentials_device_map_credentials FOREIGN KEY (credentialsid)
      REFERENCES credentials (credentialsid) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT fk_credentials_device_map_device FOREIGN KEY (deviceid)
      REFERENCES infrastructure_device (deviceid) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);

--DROP TABLE device_software;
CREATE TABLE device_software
(
  deviceid  		bigint NOT NULL, -- infrastructure device id from device
  instanceid  		character varying(255) NULL,
  name 				character varying(255) NOT NULL,
  version 			character varying(128) NOT NULL,
  installationdate timestamp,
  softwaretype bigint NOT NULL,
  manufacturer		character varying(255) NOT NULL,
  device_id			character varying(128) , -- this the phyical device id
  subdeviceid		character varying(128),
  vendorid			character varying(128),
  subvendorid		character varying(128),
  componentid	    character varying(128),
  CONSTRAINT device_software_pkey PRIMARY KEY (deviceid, instanceid),
  CONSTRAINT fk_device_software_device FOREIGN KEY (deviceid)
      REFERENCES infrastructure_device (deviceid) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE,
 CONSTRAINT fk_device_software_type FOREIGN KEY (softwaretype)
      REFERENCES software_type (softwaretype) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE io_module
(
  iomoduleid bigint NOT NULL,
  role character varying(255),
  CONSTRAINT io_module_pkey PRIMARY KEY (iomoduleid),
  CONSTRAINT fk_chassis_io_module_device FOREIGN KEY (iomoduleid)
  REFERENCES infrastructure_device (deviceid) MATCH SIMPLE
  ON UPDATE CASCADE ON DELETE CASCADE
  
);

